package de.zkm.pdf;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;

import org.apache.pdfbox.Loader;
import org.apache.pdfbox.pdmodel.PDDocument;

import de.zkm.pdf.model.PageTextBoxData;
import de.zkm.pdf.process.PageTextBoxDataProcessor;
import de.zkm.pdf.text.PositionalPDFTextStripper;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class PdfTextExtractor {
	
	int dpi = 300;

	public float dpi2UserUnit(int dpi) {
		float unitDpi = 72.0f;
		return (float) dpi / unitDpi;
	}
	
	public PdfTextExtractor(int dpi) {
		this.dpi = dpi;
	}

	/**
	 * 
	 * @param pdfFileUri e.g.: file://./testdata/ZKM-01-0122-02-00051.pdf"
	 */
	public List<PageTextBoxData> extractWords(URI pdfFileUri) {

		try {
			PDDocument document = loadDocument(pdfFileUri);
			PositionalPDFTextStripper stripper = new PositionalPDFTextStripper();
			List<PageTextBoxData> pageTextBoxDataList = stripper.getPageData(pdfFileUri, document);
			document.close();
			return pageTextBoxDataList;
		} catch (IOException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	/**
	 * 
	 * @param pdfFileUri e.g.: file://./testdata/ZKM-01-0122-02-00051.pdf"
	 */
	public List<PageTextBoxData> extractLines(URI pdfFileUri) {

			List<PageTextBoxData> pageTextBoxDataList = extractWords(pdfFileUri);
		
			PageTextBoxDataProcessor processor = new PageTextBoxDataProcessor();
			for(PageTextBoxData bd : pageTextBoxDataList){
				log.debug("=====================================");
				log.debug(Integer.toString(bd.getPageIndex()));
				bd.getTextBoxes().stream().forEach(tb -> log.debug("    " + tb.toString()));
				PageTextBoxData pageTextLines = processor.joinWordsToLines(bd);
				pageTextLines.getTextBoxes().stream().forEach(tb -> log.debug("==== " + tb.toString()));
			}
			return pageTextBoxDataList;
	}

	protected PDDocument loadDocument(URI pdfFileUri) throws IOException {
		File pdfFile = new File(pdfFileUri); 
		float customUnit = dpi2UserUnit(dpi);
		PDDocument document = Loader.loadPDF(pdfFile);
		for (int i = 0; i < document.getNumberOfPages(); i++) {
			document.getPage(i).setUserUnit(customUnit);
		}
		return document;
	}
	
	public static void main(String[] args) throws URISyntaxException {
		PdfTextExtractor extractor = new PdfTextExtractor(300);
		extractor.extractLines(new URI(args[0]));
	}
}
