package de.zkm.pdf.experimental;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.imageio.spi.IIORegistry;
import javax.imageio.spi.ImageReaderSpi;

import org.apache.pdfbox.Loader;
import org.apache.pdfbox.cos.COSName;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDResources;
import org.apache.pdfbox.pdmodel.graphics.PDXObject;
import org.apache.pdfbox.pdmodel.graphics.image.PDImageXObject;
import org.apache.pdfbox.rendering.ImageType;
import org.apache.pdfbox.rendering.PDFRenderer;

import com.github.jaiimageio.jpeg2000.impl.J2KImageReaderSpi;


public class ExtractPageImages {

    public static void main(String[] args) {
        try {
        	// Register the JPEG2000 ImageReaderSPI
        	// Register the JPEG2000 ImageReaderSpi
            IIORegistry registry = IIORegistry.getDefaultInstance();
            ImageReaderSpi readerSpi = new J2KImageReaderSpi();
            registry.registerServiceProvider(readerSpi);

            File pdfFile = new File("testdata/ZKM-01-0122-02-00051.pdf"); // Replace with the path to your PDF file

            PDDocument document = Loader.loadPDF(pdfFile);
            PDFRenderer pdfRenderer = new PDFRenderer(document);

            for (int pageNumber = 0; pageNumber < document.getNumberOfPages(); pageNumber++) {
                PDPage page = document.getPage(pageNumber);
                
                // Render the page as an image
                BufferedImage image = pdfRenderer.renderImageWithDPI(pageNumber, 300, ImageType.RGB);

                // Get image dimensions
                int width = image.getWidth();
                int height = image.getHeight();

                // Save the image as a file (you can choose the image format)
                File outputImage = new File("page_" + (pageNumber + 1) + ".png");
                ImageIO.write(image, "PNG", outputImage);

                System.out.println("Page " + (pageNumber + 1) + " - Image Dimensions:");
                System.out.println("Width: " + width + " pixels");
                System.out.println("Height: " + height + " pixels");
                System.out.println("Image saved as " + outputImage.getName());
            }

            document.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}