package de.zkm.pdf.experimental;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.pdfbox.Loader;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.text.PDFTextStripper;
import org.apache.pdfbox.text.TextPosition;

import de.zkm.pdf.model.WordBounds;

/**
 * Experiment with pdfbox in order to extract wordbounds from PDFs.
 * 
 * see also https://github.com/apache/pdfbox/blob/trunk/examples/src/main/java/org/apache/pdfbox/examples/util/PrintTextLocations.java
 * 
 * 
 */
public class PdfWordBounds {
	
	public static float dpi2UserUnit(int dpi) {
		float unitDpi = 72.0f;
		return (float)dpi / unitDpi;
	}

    public static void main(String[] args) {
        try {
            File pdfFile = new File("testdata/ZKM-01-0122-02-00051.pdf"); // Replace with the path to your PDF file

            float customUnit = dpi2UserUnit(300);
            
            PDDocument document = Loader.loadPDF(pdfFile);
           
            for (int i = 0; i < document.getNumberOfPages(); i++) {
                document.getPage(i).setUserUnit(customUnit);
            }
            
            PDFTextStripper stripper = new PDFTextStripper() {
            	
                float pageUserUnit = 0f;
                
                @Override
                public void processPage(PDPage page) throws IOException {
                	super.processPage(page);
                	this.pageUserUnit = page.getUserUnit();
                }
                
                protected float scale(float value) {
                	return value * pageUserUnit;
                }
                
                protected WordBounds claculateBounds(List<TextPosition> textPositions) {
                	WordBounds wb  = new WordBounds();
                	if(textPositions.size() > 0) {
                		wb.x = textPositions.get(0).getXDirAdj();
                		wb.y = textPositions.get(0).getYDirAdj();
                		for(TextPosition pos : textPositions) {
                			wb.w = wb.w + pos.getWidthDirAdj();
                			wb.h = Math.max(wb.h, pos.getHeightDir());
                			wb.word.append(pos.getUnicode());
                		}
                	}
                	wb.x = scale(wb.x);
                	wb.y = scale(wb.y);
                	wb.w = scale(wb.w);
                	wb.h = scale(wb.h);
                	return wb;
                }
                
                @Override
                protected void writeString(String text, List<TextPosition> textPositions) throws IOException {
                	List<TextPosition> wordPositions = new ArrayList<TextPosition>();

                    System.out.println("=========================================");
                    System.out.println(text);
                    for (TextPosition position : textPositions)
                    {
                        System.out.println( "String[" + 
                        		scale(position.getXDirAdj()) + "," +
                        		scale(position.getYDirAdj()) + 
                        		" fs=" + position.getFontSize() +
                        		" xscale=" + position.getXScale() +
                        		" height=" + scale(position.getHeightDir()) +
                        		" space=" + scale(position.getWidthOfSpace()) + 
                        		" width=" + scale(position.getWidthDirAdj()) + 
                        		"]" + position.getUnicode() );
                        if(position.getUnicode().equals(getWordSeparator())) {
                        	System.out.println("---------------- WORDSEP ----------------");
                        	WordBounds wb = claculateBounds(wordPositions);
                        	System.out.println(wb.x + ", " + wb.y + ", " + wb.w + ", " + wb.h + ": " + wb.word.toString());
                        	System.out.println("-----------------------------------------");
                        	wordPositions.clear();
                        	// 
                        } else if(position.getUnicode().matches("\\p{Punct}")) {
                        	System.out.println("---------------- Punct ----------------");
                        } else {
                        	wordPositions.add(position);
                        }
                    }
                    
//                    for (TextPosition position : textPositions) {
//                        // Check if the text position is a space, indicating a word boundary
//                        if (position.getUnicode().equals(" ")) {
//                            if (word.length() > 0) {
//                                // Output the word
//                                System.out.println(word.toString());
//                                word.setLength(0); // Clear the word buffer
//                            }
//                        } else {
//                            word.append(position.getUnicode());
//                        }
//                    }
//
//                    // Output the last word if there's no space at the end
//                    if (word.length() > 0) {
//                        System.out.println(word.toString());
//                    }
                }
            };

            stripper.setSortByPosition( true );
            stripper.setWordSeparator(" ");
            stripper.setStartPage( 0 );
            stripper.setEndPage( document.getNumberOfPages() );
            
            String text = stripper.getText(document);
            System.out.println("Extracted text:\n" + text);

            document.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}

